"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mysql_1 = require("mysql");
const map = new Map([
    ["$not", "<>"],
    ["$gt", ">"],
    ["$gte", ">="],
    ["$lt", "<"],
    ["$lte", "<="],
    ["$in", "IN"],
    ["$notIn", "NOT IN"],
    ["$like", "LIKE"],
    ["$notLike", "NOT LIKE"],
    ["$isNull", "IS NULL"],
    ["_isNotNull", "IS NOT NULL"],
]);
function parseConditionObject(condition) {
    if (!condition) {
        return [];
    }
    return Object.keys(condition).map((key) => {
        const value = condition[key];
        if (typeof value === "object") {
            let [opr] = Object.keys(value).filter((k) => map.has(k));
            if (opr) {
                let resultValue = mysql_1.escape(value[opr]);
                if ((opr === "$in" || opr === "$notIn")) {
                    if (!resultValue) {
                        return "";
                    }
                    resultValue = `(${resultValue})`;
                }
                else if (opr === "$isNull") {
                    if (resultValue === "false") {
                        opr = "_isNotNull";
                    }
                    resultValue = "";
                }
                return `${mysql_1.escapeId(key)} ${map.get(opr)} ${resultValue}`;
            }
        }
        else {
            return `${mysql_1.escapeId(key)} = ${mysql_1.escape(value)}`;
        }
    })
        .map((x) => x.trim());
}
function buildWhere(...args) {
    if (typeof args[0] === "string") {
        return `WHERE ${mysql_1.format(args[0], args[1])}`;
    }
    else {
        const ors = args
            .map((arg) => {
            const ands = parseConditionObject(arg);
            if (ands.length === 1 || args.length === 1) {
                return ands.join(" AND ");
            }
            else {
                return `(${ands.join(" AND ")})`;
            }
        })
            .filter((x) => !!x);
        if (!ors.length) {
            return "";
        }
        return "WHERE " + ors.join(" OR ");
    }
}
exports.buildWhere = buildWhere;
function buildLimit(limit) {
    return `LIMIT ${mysql_1.escape(limit)}`;
}
exports.buildLimit = buildLimit;
function buildOffset(offset) {
    return `OFFSET ${mysql_1.escape(offset)}`;
}
exports.buildOffset = buildOffset;
function buildOrderBy(...orders) {
    const orderSql = orders.map(([key, sort]) => `${mysql_1.escapeId(key)} ${sort > 0 ? "ASC" : "DESC"}`).join(", ");
    return `ORDER BY ${orderSql}`;
}
exports.buildOrderBy = buildOrderBy;
function buildGroupBy(column) {
    return `GROUP BY ${mysql_1.escapeId(column)}`;
}
exports.buildGroupBy = buildGroupBy;
function joinSql(list) {
    return list.filter((x) => !!x).join(" ");
}
exports.joinSql = joinSql;
//# sourceMappingURL=builder.js.map