"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_batch_1 = require("./query_batch");
const query_delete_1 = require("./query_delete");
const query_insert_1 = require("./query_insert");
const query_select_1 = require("./query_select");
const query_update_1 = require("./query_update");
class Client {
    constructor(queryFn) {
        this.queryFn = queryFn;
    }
    select(...columns) {
        const select = new query_select_1.QuerySelect(this.queryFn);
        return select.select(...columns);
    }
    count(column = '*') {
        const select = new query_select_1.QuerySelect(this.queryFn);
        select.plugin((result) => result[0].count);
        return select.select(`COUNT(${column}) AS count`);
    }
    insert(values) {
        const insert = new query_insert_1.QueryInsert(this.queryFn);
        return insert.values(values);
    }
    delete(filter) {
        const del = new query_delete_1.QueryDelete(this.queryFn);
        return del.where(filter);
    }
    update(table) {
        const update = new query_update_1.QueryUpdate(this.queryFn);
        return update.update(table);
    }
    batch(table) {
        const batch = new query_batch_1.QueryBatch(this.queryFn);
        return batch.batch(table);
    }
    query(options, values) {
        return new Promise((resolve, reject) => {
            this.queryFn(options, values, (err, result) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(result);
                }
            });
        });
    }
}
exports.Client = Client;
//# sourceMappingURL=client.js.map