"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mysql_1 = require("mysql");
const bsql_error_1 = require("./bsql_error");
const query_1 = require("./query");
const transform_1 = require("./transform");
class QueryBatch extends query_1.Query {
    batch(table) {
        this.sql.update = `UPDATE ${table}`;
        return this;
    }
    update(rows, matchColumn = 'id') {
        const lines = [];
        const columns = new Map();
        transform_1.snakeCase(rows).forEach((v) => {
            Object.keys(v).forEach((k) => {
                if (k !== matchColumn) {
                    const column = columns.get(k) || [];
                    column.push(mysql_1.format(`WHEN ?? = ? THEN ?`, [matchColumn, v[matchColumn], v[k]]));
                    columns.set(k, column);
                }
            });
        });
        columns.forEach((v, k) => {
            lines.push(`${mysql_1.escapeId(k)} = CASE ${v.join(' ')} ELSE ${mysql_1.escapeId(k)} END`);
        });
        this.sql.set = `SET ${lines.join(' , ')}`;
        return this;
    }
    getSql() {
        bsql_error_1.assert(this.sql.update, 'Table name is required, please call batch(table).');
        bsql_error_1.assert(this.sql.set, 'Values is required, please call update(values).');
        return [this.sql.update, this.sql.set];
    }
}
exports.QueryBatch = QueryBatch;
//# sourceMappingURL=query_batch.js.map