"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mysql_1 = require("mysql");
const bsql_error_1 = require("./bsql_error");
const query_1 = require("./query");
const transform_1 = require("./transform");
class QueryInsert extends query_1.Query {
    into(table) {
        this.sql.insert = `INSERT INTO ${mysql_1.escapeId(table)}`;
        return this;
    }
    values(values) {
        const formatted = transform_1.snakeCase(values);
        const keys = [];
        formatted.forEach((value) => Object.keys(value).forEach((k) => {
            if (keys.indexOf(k) === -1) {
                keys.push(k);
            }
        }));
        this.sql.column = `(${keys.map((k) => mysql_1.escapeId(k)).join(',')})`;
        this.sql.values = 'VALUES' + formatted.map((value) => {
            const arr = new Array(keys.length);
            Object.keys(value).forEach((k) => {
                arr[keys.indexOf(k)] = value[k];
            });
            return `(${mysql_1.escape(arr)})`;
        }).join(',');
        return this;
    }
    onDuplicateKey(sql) {
        this.sql.duplicate = `ON DUPLICATE KEY ${sql}`;
        return this;
    }
    getSql() {
        bsql_error_1.assert(this.sql.insert, 'Table name is required, please call into(table).');
        bsql_error_1.assert(this.sql.column, 'Values is required, please call values(values).');
        return [this.sql.insert, this.sql.column, this.sql.values, this.sql.duplicate];
    }
}
exports.QueryInsert = QueryInsert;
//# sourceMappingURL=query_insert.js.map