"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mysql_1 = require("mysql");
const bsql_error_1 = require("./bsql_error");
const query_1 = require("./query");
const transform_1 = require("./transform");
const where_filter_1 = require("./where_filter");
var OrderType;
(function (OrderType) {
    OrderType[OrderType["Asc"] = 1] = "Asc";
    OrderType[OrderType["Desc"] = -1] = "Desc";
})(OrderType || (OrderType = {}));
class QuerySelect extends query_1.Query {
    constructor(query) {
        super(query);
        this.plugin((results) => transform_1.camelCase(results));
    }
    select(...columns) {
        if (!columns || !columns.length || columns[0] === '*') {
            this.sql.select = 'SELECT *';
        }
        else {
            const formatted = columns.map((c) => {
                const match = /^(\w+)\((.+)\)(.*)/.exec(c);
                if (!match) {
                    return mysql_1.escapeId(transform_1.snakeCase(c));
                }
                else {
                    return `${match[1]}(${match[2] === '*' ? '*' : mysql_1.escapeId(transform_1.snakeCase(match[2]))})${match[3] || ''}`;
                }
            });
            this.sql.select = `SELECT ${formatted.join(',')}`;
        }
        return this;
    }
    from(table) {
        this.sql.from = `FROM ${mysql_1.escapeId(table)}`;
        return this;
    }
    where(filter) {
        this.sql.where = `WHERE ${where_filter_1.buildWhereSql(filter)}`;
        return this;
    }
    groupBy(column) {
        this.sql.groupBy = `GROUP BY ${mysql_1.escapeId(transform_1.snakeCase(column))}`;
        return this;
    }
    having(filter) {
        this.sql.having = `HAVING ${where_filter_1.buildWhereSql(filter)}`;
        return this;
    }
    orderBy(...order) {
        this.sql.orderBy = 'ORDER BY '
            + order.map(([k, v]) => `${mysql_1.escapeId(k)} ${v === OrderType.Asc ? 'ASC' : 'DESC'}`).join(',');
        return this;
    }
    limit(num) {
        this.sql.limit = `LIMIT ${mysql_1.escape(num)}`;
        return this;
    }
    offset(num) {
        this.sql.offset = `OFFSET ${mysql_1.escape(num)}`;
        return this;
    }
    getSql() {
        bsql_error_1.assert(this.sql.from, 'Table name is required, please call from(table).');
        return [this.sql.select, this.sql.from, this.sql.where,
            this.sql.groupBy, this.sql.having, this.sql.orderBy, this.sql.limit, this.sql.offset];
    }
}
exports.QuerySelect = QuerySelect;
//# sourceMappingURL=query_select.js.map