"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function camelCase(value) {
    return transform(value, (str) => str.replace(/_([a-z])/g, ($1, $2) => $2.toUpperCase()));
}
exports.camelCase = camelCase;
function snakeCase(value) {
    return transform(value, (str) => str.replace(/^[A-Z]/, ($1) => $1.toLowerCase())
        .replace(/[A-Z][a-z]/g, ($1) => '_' + $1.toLowerCase()));
}
exports.snakeCase = snakeCase;
function transform(value, func) {
    const type = typeof value;
    if (type === 'string') {
        return func(value);
    }
    else if (type === 'object') {
        if (value instanceof Array) {
            return value.map((v) => transform(v, func));
        }
        else {
            const tmp = {};
            Object.keys(value).forEach((k) => {
                tmp[func(k)] = value[k];
            });
            return tmp;
        }
    }
    return value;
}
//# sourceMappingURL=transform.js.map