"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mysql_1 = require("mysql");
const transform_1 = require("./transform");
function buildWhereSql(filter) {
    return Object.keys(filter).map((key) => {
        if (key === '$or') {
            return filter.$or.map(buildWhereSql).join(' OR ');
        }
        else {
            return buildOpr(key, filter[key]);
        }
    }).join(' AND ');
}
exports.buildWhereSql = buildWhereSql;
function buildOpr(key, opr) {
    const type = typeof opr;
    const left = mysql_1.escapeId(transform_1.snakeCase(key));
    let operator;
    let right;
    if (type === 'number' || type === 'string' || type === 'boolean') {
        operator = '=';
        right = mysql_1.escape(opr);
    }
    else if (type === 'object') {
        if ('$gt' in opr) {
            operator = '>';
            right = mysql_1.escape(opr.$gt);
        }
        else if ('$lt' in opr) {
            operator = '<';
            right = mysql_1.escape(opr.$lt);
        }
        else if ('$not' in opr) {
            operator = '<>';
            right = mysql_1.escape(opr.$not);
        }
        else if ('$gte' in opr) {
            operator = '>=';
            right = mysql_1.escape(opr.$gte);
        }
        else if ('$lte' in opr) {
            operator = '<=';
            right = mysql_1.escape(opr.$lte);
        }
        else if ('$like' in opr) {
            operator = 'LIKE';
            right = mysql_1.escape(opr.$like);
        }
        else if ('$notLike' in opr) {
            operator = 'NOT LIKE';
            right = mysql_1.escape(opr.$notLike);
        }
        else if ('$in' in opr) {
            operator = 'IN';
            right = `(${mysql_1.escape(opr.$in)})`;
        }
        else if ('$notIn' in opr) {
            operator = 'NOT IN';
            right = `(${mysql_1.escape(opr.$notIn)})`;
        }
        else if ('$isNull' in opr) {
            operator = opr.$isNull ? 'IS NULL' : 'IS NOT NULL';
        }
    }
    return [left, operator, right].filter((y) => !!y).join(' ');
}
//# sourceMappingURL=where_filter.js.map